@@ -152,6 +152,10 @@ static PixelBuffer *GetCompositePixelBuffer(GLenum format, GLenum type)
       (FlGui::instance()->getCurrentOpenglWindow()->getDrawContext());
     newg->show();
     openglWindow::setLastHandled(newg);
+    // waiting for the OS to really make the window visible and to call the
+    // draw() function on (some ?) linux; if we do not wait here, the window is
+    // not ready and the picture cannot be generated
+    while(!newg->valid()) Fl::wait();
   }
 
   PixelBuffer *buffer;
@@ -303,7 +307,8 @@ void CreateOutputFile(const std::string &fileName, int format, bool redraw)
 
   case FORMAT_INP:
     GModel::current()->writeINP
-      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.saveGroupsOfNodes,
+       CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_P3D:
@@ -353,7 +358,7 @@ void CreateOutputFile(const std::string &fileName, int format, bool redraw)
     {
       if(!FlGui::available()) break;
 
-      FILE *fp = fopen(name.c_str(), "wb");
+      FILE *fp = Fopen(name.c_str(), "wb");
       if(!fp){
         Msg::Error("Unable to open file '%s'", name.c_str());
         error = true;
@@ -390,7 +395,7 @@ void CreateOutputFile(const std::string &fileName, int format, bool redraw)
     {
       if(!FlGui::available()) break;
 
-      FILE *fp = fopen(name.c_str(), "wb");
+      FILE *fp = Fopen(name.c_str(), "wb");
       if(!fp){
         Msg::Error("Unable to open file '%s'", name.c_str());
         error = true;
@@ -461,7 +466,7 @@ void CreateOutputFile(const std::string &fileName, int format, bool redraw)
     {
       if(!FlGui::available()) break;
 
-      FILE *fp = fopen(name.c_str(), "w");
+      FILE *fp = Fopen(name.c_str(), "w");
       if(!fp){
         Msg::Error("Unable to open file '%s'", name.c_str());
         error = true;
@@ -493,7 +498,7 @@ void CreateOutputFile(const std::string &fileName, int format, bool redraw)
   case FORMAT_MPEG:
     {
       std::string parFileName = CTX::instance()->homeDir + ".gmsh-mpeg_encode.par";
-      FILE *fp = fopen(parFileName.c_str(), "w");
+      FILE *fp = Fopen(parFileName.c_str(), "w");
       if(!fp){
         Msg::Error("Unable to open file '%s'", parFileName.c_str());
         error = true;
