@@ -100,6 +100,7 @@ void SetBoundingBox(double xmin, double xmax,
                              SQU(CTX::instance()->max[2] - CTX::instance()->min[2]));
   for(int i = 0; i < 3; i++)
     CTX::instance()->cg[i] = 0.5 * (CTX::instance()->min[i] + CTX::instance()->max[i]);
+
 }
 
 void SetBoundingBox(bool aroundVisible)
@@ -116,7 +117,6 @@ void SetBoundingBox(bool aroundVisible)
           bb += PView::list[i]->getData()->getBoundingBox();
   }
 #endif
-
   if(bb.empty()){
     bb += SPoint3(-1., -1., -1.);
     bb += SPoint3(1., 1., 1.);
@@ -131,13 +131,13 @@ void SetBoundingBox(bool aroundVisible)
                              SQU(CTX::instance()->max[2] - CTX::instance()->min[2]));
   for(int i = 0; i < 3; i++)
     CTX::instance()->cg[i] = 0.5 * (CTX::instance()->min[i] + CTX::instance()->max[i]);
+
 }
 
-// FIXME: this is necessary for now to have an approximate
-// CTX::instance()->lc *while* parsing input files (it's important
-// since some of the geometrical operations use a tolerance that
-// depends on CTX::instance()->lc). This will be removed once the new
-// database is filled directly during the parsing step
+// FIXME: this is necessary for now to have an approximate CTX::instance()->lc
+// *while* parsing input files (it's important since some of the geometrical
+// operations use a tolerance that depends on CTX::instance()->lc). This will be
+// removed once the new database is filled directly during the parsing step
 static SBoundingBox3d temp_bb;
 
 void ResetTemporaryBoundingBox()
@@ -185,7 +185,7 @@ int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
   // add 'b' for pure Windows programs: opening in text mode messes up
   // fsetpos/fgetpos (used e.g. for user-defined functions)
   FILE *fp;
-  if(!(fp = fopen(fileName.c_str(), "rb"))){
+  if(!(fp = Fopen(fileName.c_str(), "rb"))){
     if(warnIfMissing)
       Msg::Warning("Unable to open file '%s'", fileName.c_str());
     return 0;
@@ -233,7 +233,7 @@ int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
 
 #if defined(HAVE_FLTK) && defined(HAVE_POST)
   if(FlGui::available())
-    FlGui::instance()->updateViews(numViewsBefore != (int)PView::list.size());
+    FlGui::instance()->updateViews(numViewsBefore != (int)PView::list.size(), false);
 #endif
 
   return 1;
@@ -244,7 +244,7 @@ void ParseString(const std::string &str)
 {
   if(str.empty()) return;
   std::string fileName = CTX::instance()->homeDir + CTX::instance()->tmpFileName;
-  FILE *fp = fopen(fileName.c_str(), "w");
+  FILE *fp = Fopen(fileName.c_str(), "w");
   if(fp){
     fprintf(fp, "%s\n", str.c_str());
     fclose(fp);
@@ -252,6 +252,16 @@ void ParseString(const std::string &str)
   }
 }
 
+static int defineSolver(const std::string &name)
+{
+  for(int i = 0; i < NUM_SOLVERS; i++){
+    if(opt_solver_name(i, GMSH_GET, "") == name) return i;
+  }
+  // overwrite last one
+  opt_solver_name(NUM_SOLVERS - 1, GMSH_SET|GMSH_GUI, name);
+  return NUM_SOLVERS - 1;
+}
+
 int MergeFile(const std::string &fileName, bool warnIfMissing)
 {
   if(GModel::current()->getName() == ""){
@@ -266,7 +276,7 @@ int MergeFile(const std::string &fileName, bool warnIfMissing)
 
   // added 'b' for pure Windows programs, since some of these files
   // contain binary data
-  FILE *fp = fopen(fileName.c_str(), "rb");
+  FILE *fp = Fopen(fileName.c_str(), "rb");
   if(!fp){
     if(warnIfMissing)
       Msg::Warning("Unable to open file '%s'", fileName.c_str());
@@ -274,7 +284,7 @@ int MergeFile(const std::string &fileName, bool warnIfMissing)
   }
 
   char header[256];
-  if(!fgets(header, sizeof(header), fp)) return 0;
+  if(!fgets(header, sizeof(header), fp)){ fclose(fp); return 0; }
   fclose(fp);
 
   Msg::StatusBar(true, "Reading '%s'...", fileName.c_str());
@@ -402,13 +412,20 @@ int MergeFile(const std::string &fileName, bool warnIfMissing)
     status = readFile3M(fileName);
   }
 #endif
-#if defined(HAVE_ONELAB) && defined(HAVE_FLTK)
-  else if(ext == ".pro" && opt_solver_name(0, GMSH_GET, "") == "GetDP"){
-    std::vector<std::string> split = SplitFileName(fileName);
-    GModel::current()->setName("");
-    status = MergeFile(split[0] + split[1] + ".geo");
-    CTX::instance()->launchSolverAtStartup = 0;
-    return status;
+#if defined(HAVE_ONELAB)
+  else if(ext == ".pro" || ext == ".PRO"){
+    int num = defineSolver("GetDP");
+    GModel::current()->setName(split[1] + ".geo");
+    GModel::current()->setFileName(split[0] + split[1] + ".geo");
+    CTX::instance()->launchSolverAtStartup = num;
+    return 1;
+  }
+  else if(ext == ".py" || ext == ".PY" ||
+          ext == ".exe" || ext == ".EXE"){
+    int num = defineSolver(split[1]);
+    opt_solver_executable(num, GMSH_SET, fileName);
+    CTX::instance()->launchSolverAtStartup = num;
+    return 1;
   }
 #endif
 #if defined(HAVE_ONELAB_METAMODEL) && defined(HAVE_FLTK)
@@ -416,11 +433,6 @@ int MergeFile(const std::string &fileName, bool warnIfMissing)
     // FIXME: this is a hack -- think about a better way
     status = metamodel_cb(fileName);
   }
-  else if(ext == ".py"){
-    FlGui::instance()->onelab->addSolver("python", fileName, "", 1);
-    onelab_cb(0, (void*)"check");
-    status = 1;
-  }
 #endif
   else {
     CTX::instance()->geom.draw = 1;
@@ -462,7 +474,6 @@ int MergeFile(const std::string &fileName, bool warnIfMissing)
 
   ComputeMaxEntityNum();
   SetBoundingBox();
-
   CTX::instance()->geom.draw = 1;
   CTX::instance()->mesh.changed = ENT_ALL;
 
@@ -472,7 +483,7 @@ int MergeFile(const std::string &fileName, bool warnIfMissing)
     for(unsigned int i = numViewsBefore; i < PView::list.size(); i++)
       opt_view_timestep(i, GMSH_SET | GMSH_GUI,
                         PView::list[i]->getData()->getFirstNonEmptyTimeStep());
-    FlGui::instance()->updateViews(numViewsBefore != (int)PView::list.size());
+    FlGui::instance()->updateViews(numViewsBefore != (int)PView::list.size(), false);
   }
 #endif
 
@@ -480,7 +491,7 @@ int MergeFile(const std::string &fileName, bool warnIfMissing)
 
   if(!status) Msg::Error("Error loading '%s'", fileName.c_str());
   Msg::StatusBar(true, "Done reading '%s'", fileName.c_str());
-
+  CTX::instance()->fileread=true;
    // merge the associated option file if there is one
   if(!StatFile(fileName + ".opt"))
     MergeFile(fileName + ".opt");
@@ -493,13 +504,13 @@ int MergePostProcessingFile(const std::string &fileName, bool showLastStep,
 {
 #if defined(HAVE_POST)
   // check if there is a mesh in the file
-  FILE *fp = fopen(fileName.c_str(), "rb");
+  FILE *fp = Fopen(fileName.c_str(), "rb");
   if(!fp){
     if(warnIfMissing) Msg::Warning("Unable to open file '%s'", fileName.c_str());
     return 0;
   }
   char header[256];
-  if(!fgets(header, sizeof(header), fp)) return 0;
+  if(!fgets(header, sizeof(header), fp)){ fclose(fp); return 0; }
   bool haveMesh = false;
   if(!strncmp(header, "$MeshFormat", 11)){
     while(!feof(fp) && fgets(header, sizeof(header), fp)){
@@ -542,7 +553,7 @@ int MergePostProcessingFile(const std::string &fileName, bool showLastStep,
         PView::list[i]->getOptions()->visible = 0;
     }
   }
-  else if(n != PView::list.size()){
+  else if(n < PView::list.size()){
     // if we created new views, assume we only want to see those (and the
     // onelab X-Y graphs)
     for(unsigned int i = 0; i < n; i++){
@@ -595,7 +606,7 @@ void ClearProject()
   if(FlGui::available()){
     FlGui::instance()->setGraphicTitle(GModel::current()->getFileName());
     FlGui::instance()->resetVisibility();
-    FlGui::instance()->updateViews();
+    FlGui::instance()->updateViews(true, true);
     FlGui::instance()->updateFields();
     GModel::current()->setSelection(0);
   }
@@ -614,17 +625,21 @@ void OpenProject(const std::string &fileName)
   Msg::ResetErrorCounter();
 
   if(GModel::current()->empty()){
-    // if the current model is empty, make sure it's reaaally
-    // cleaned-up, and reuse it (don't clear the parser variables: if
-    // the model is empty we probably just launched gmsh, and we don't
-    // want to delete variables set e.g. using the -string command
-    // line option)
+    // if the current model is empty, make sure it's reaaally cleaned-up, and
+    // reuse it
     GModel::current()->destroy();
     GModel::current()->getGEOInternals()->destroy();
+    // don't clear the parser variables if we just launched gmsh with the
+    // -string command line option
+#if defined(HAVE_PARSER)
+    std::string c = Msg::GetCommandLineArgs();
+    if(c.find("-string") == std::string::npos)
+      gmsh_yysymbols.clear();
+#endif
   }
   else{
-    // if the current model is not empty make it invisible, clear the
-    // parser variables and add a new model
+    // if the current model is not empty make it invisible, clear the parser
+    // variables and add a new model
 #if defined(HAVE_PARSER)
     gmsh_yysymbols.clear();
 #endif
@@ -632,8 +647,7 @@ void OpenProject(const std::string &fileName)
     GModel::current(GModel::list.size() - 1);
   }
 
-  // temporary hack until we fill the current GModel on the fly during
-  // parsing
+  // temporary hack until we fill the current GModel on the fly during parsing
   ResetTemporaryBoundingBox();
 
   // merge the file
@@ -662,7 +676,7 @@ void OpenProject(const std::string &fileName)
   if(FlGui::available()){
     file_watch_cb(0, 0);
     FlGui::instance()->resetVisibility();
-    FlGui::instance()->updateViews();
+    FlGui::instance()->updateViews(true, false);
     FlGui::instance()->updateFields();
     GModel::current()->setSelection(0);
     GModel::current()->setCompoundVisibility();
@@ -681,6 +695,8 @@ void OpenProjectMacFinder(const char *fileName)
     // Gmsh is running
     OpenProject(fileName);
     drawContext::global()->draw();
+    if(CTX::instance()->launchSolverAtStartup >= 0)
+      solver_cb(0, (void*)CTX::instance()->launchSolverAtStartup);
   }
 #endif
 }
