@@ -156,6 +156,8 @@ double opt_general_clip_position0(OPT_ARGS_NUM);
 double opt_general_clip_position1(OPT_ARGS_NUM);
 double opt_general_manip_position0(OPT_ARGS_NUM);
 double opt_general_manip_position1(OPT_ARGS_NUM);
+double opt_general_hot_position0(OPT_ARGS_NUM);
+double opt_general_hot_position1(OPT_ARGS_NUM);
 double opt_general_session_save(OPT_ARGS_NUM);
 double opt_general_options_save(OPT_ARGS_NUM);
 double opt_general_rotation0(OPT_ARGS_NUM);
@@ -444,9 +446,10 @@ double opt_mesh_allow_swap_edge_angle(OPT_ARGS_NUM);
 double opt_mesh_min_curv_points(OPT_ARGS_NUM);
 double opt_mesh_multiple_passes(OPT_ARGS_NUM);
 double opt_mesh_order(OPT_ARGS_NUM);
-double opt_mesh_smooth_internal_edges(OPT_ARGS_NUM);
+double opt_mesh_ho_optimize(OPT_ARGS_NUM);
 double opt_mesh_ho_nlayers(OPT_ARGS_NUM);
-double opt_mesh_ho_mindisto(OPT_ARGS_NUM);
+double opt_mesh_ho_threshold_min(OPT_ARGS_NUM);
+double opt_mesh_ho_threshold_max(OPT_ARGS_NUM);
 double opt_mesh_ho_poisson(OPT_ARGS_NUM);
 double opt_mesh_second_order_experimental(OPT_ARGS_NUM);
 double opt_mesh_second_order_linear(OPT_ARGS_NUM);
