@@ -4,11 +4,6 @@
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include "GmshConfig.h"
-#if !defined(HAVE_NO_STDINT_H)
-#include <stdint.h>
-#elif defined(HAVE_NO_INTPTR_T)
-typedef unsigned long intptr_t;
-#endif
 #include <string>
 #include <sstream>
 #include <map>
@@ -20,28 +15,19 @@ typedef unsigned long intptr_t;
 #include <FL/Fl_Value_Input.H>
 #include "GmshConfig.h"
 #include "FlGui.h"
+#include "graphicWindow.h"
 #include "drawContext.h"
 #include "highOrderToolsWindow.h"
 #include "paletteWindow.h"
-#include "contextWindow.h"
-#include "graphicWindow.h"
-#include "GmshDefines.h"
 #include "GmshMessage.h"
 #include "GModel.h"
 #include "MElement.h"
-#include "PView.h"
-#include "PViewData.h"
-#include "GeoStringInterface.h"
-#include "Options.h"
 #include "Context.h"
 #include "HighOrder.h"
 
 #if defined(HAVE_OPTHOM)
 #include "OptHomRun.h"
-#endif
-
-#if defined(HAVE_PARSER)
-#include "Parser.h"
+#include "OptHomElastic.h"
 #endif
 
 static void change_completeness_cb(Fl_Widget *w, void *data)
@@ -62,7 +48,6 @@ static void change_completeness_cb(Fl_Widget *w, void *data)
 
 static void highordertools_runp_cb(Fl_Widget *w, void *data)
 {
-#if defined(HAVE_OPTHOM)
   highOrderToolsWindow *o = FlGui::instance()->highordertools;
 
   int order = (int)o->value[0]->value();
@@ -79,18 +64,16 @@ static void highordertools_runp_cb(Fl_Widget *w, void *data)
   computeDistanceFromMeshToGeometry (GModel::current(), dist);
   for (std::map<GEntity*, double> ::iterator it = dist.d2.begin();
        it !=dist.d2.end();++it){
-    printf ("GEntity %d of dim %d : dist %12.5E\n",
-            it->first->tag(), it->first->dim(), it->second);
+    printf("GEntity %d of dim %d : dist %12.5E\n",
+           it->first->tag(), it->first->dim(), it->second);
   }
 
   CTX::instance()->mesh.changed |= (ENT_LINE | ENT_SURFACE | ENT_VOLUME);
   drawContext::global()->draw();
-#endif
 }
 
 static void chooseopti_cb(Fl_Widget *w, void *data)
 {
-#if defined(HAVE_OPTHOM)
   highOrderToolsWindow *o = FlGui::instance()->highordertools;
   int elastic = o->choice[2]->value();
 
@@ -111,37 +94,39 @@ static void chooseopti_cb(Fl_Widget *w, void *data)
     for (int i=9;i<=11;i++) o->value[i]->activate();
     //    o->push[1]->activate();
   }
-#endif
 }
 
 static void chooseopti_strategy(Fl_Widget *w, void *data)
 {
-#if defined(HAVE_OPTHOM)
   highOrderToolsWindow *o = FlGui::instance()->highordertools;
   if (o->choice[3]->value() == 0) for (int i=9;i<=11;i++) o->value[i]->deactivate();
   else for (int i=9;i<=11;i++) o->value[i]->activate();
-#endif
 }
 
-static void highordertools_runelas_cb(Fl_Widget *w, void *data)
+static void highordertools_runopti_cb(Fl_Widget *w, void *data)
 {
-#if defined(HAVE_OPTHOM)
   highOrderToolsWindow *o = FlGui::instance()->highordertools;
 
+  if(o->butt[3]->value())
+    FlGui::instance()->graph[0]->showMessages();
+
   bool elastic = o->choice[2]->value() == 1;
   double threshold_min = o->value[1]->value();
   bool onlyVisible = (bool)o->butt[1]->value();
   int nbLayers = (int) o->value[2]->value();
   double threshold_max = o->value[8]->value();
 
-  if(elastic)ElasticAnalogy(GModel::current(), threshold_min, onlyVisible);
-  else  {
+#if defined(HAVE_OPTHOM)
+  if(elastic){
+    ElasticAnalogy(GModel::current(), threshold_min, onlyVisible);
+  }
+  else{
     OptHomParameters p;
     p.nbLayers = nbLayers;
     p.BARRIER_MIN = threshold_min;
     p.BARRIER_MAX = threshold_max;
     p.onlyVisible = onlyVisible;
-    p.dim  = GModel::current()->getDim();//o->meshOrder;
+    p.dim = GModel::current()->getDim();
     p.itMax = (int) o->value[3]->value();
     p.optPassMax = (int) o->value[4]->value();
     p.weightFixed =  o->value[5]->value();
@@ -152,13 +137,14 @@ static void highordertools_runelas_cb(Fl_Widget *w, void *data)
     p.maxAdaptBlob = o->value[9]->value();
     p.adaptBlobLayerFact = (int) o->value[10]->value();
     p.adaptBlobDistFact = o->value[11]->value();
-    HighOrderMeshOptimizer (GModel::current(),p);
-    printf("CPU TIME = %4f seconds\n",p.CPU);
+    HighOrderMeshOptimizer(GModel::current(), p);
   }
+#else
+  Msg::Error("High-order mesh optimization requires the OPTHOM module");
+#endif
 
   CTX::instance()->mesh.changed |= (ENT_LINE | ENT_SURFACE | ENT_VOLUME);
   drawContext::global()->draw();
-#endif
 }
 
 highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
@@ -166,24 +152,31 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   FL_NORMAL_SIZE -= deltaFontSize;
 
   int width = 3 * IW + 4 * WB;
-  int height = 28 * BH;
+  int height = 24 * BH;
 
   win = new paletteWindow
-    (width, height, CTX::instance()->nonModalWindows ? true : false, "High Order Tools");
+    (width, height, CTX::instance()->nonModalWindows ? true : false, "High order tools");
   win->box(GMSH_WINDOW_BOX);
 
   int y = WB;
   int x = 2 * WB;
 
+  box[0] = new Fl_Box(x, y, width - 4 * WB, BH);
+  box[0]->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
+
+  y += BH;
+
   butt[1] = new Fl_Check_Button
-    (x,y, 1.5*IW-WB, BH, "Visible entities only");
+    (x, y, width - 4 * WB, BH, "Only apply high order tools to visible entities");
   butt[1]->type(FL_TOGGLE_BUTTON);
   butt[1]->value(1);
 
-  output[0] = new Fl_Output
-    (width/2,y, IW, BH, "CAD");
-  output[0]->align(FL_ALIGN_RIGHT);
-  output[0]->value("Available");
+  y += BH;
+
+  butt[3] = new Fl_Check_Button
+    (x, y, width - 4 * WB, BH, "Show detailed log messages");
+  butt[3]->type(FL_TOGGLE_BUTTON);
+  butt[3]->value(1);
 
   {
     y += BH / 2;
@@ -195,14 +188,14 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   {
     y += BH;
     Fl_Box *b = new Fl_Box
-      (x - WB, y, width, BH, "1. Generation of high order nodes");
+      (x - WB, y, width, BH, "1. Generation of high order vertices");
     b->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
   }
 
   y += BH;
 
   value[0] = new Fl_Value_Input
-    (x,y, IW, BH, "Polynomial order");
+    (x, y, IW, BH, "Polynomial order");
   value[0]->minimum(1);
   value[0]->maximum(10);
   value[0]->step(1);
@@ -212,7 +205,7 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   y += BH;
 
   butt[0] = new Fl_Check_Button
-    (x,y, 1.5*IW-WB, BH, "Use incomplete elements");
+    (x, y, width - 4 * WB, BH, "Use incomplete elements");
   butt[0]->type(FL_TOGGLE_BUTTON);
   butt[0]->value(!complete);
   butt[0]->callback(change_completeness_cb);
@@ -220,12 +213,14 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   y += BH;
 
   butt[2] = new Fl_Check_Button
-    (x,y, 1.5*IW-WB, BH, "Generate curvilinear elements");
+    (x, y, width - 4 * WB, BH, "Generate curvilinear elements");
   butt[2]->type(FL_TOGGLE_BUTTON);
   butt[2]->value(1);
 
+  y += BH;
+
   push[0] = new Fl_Button
-    (width - BB - 2 * WB, y, BB, BH, "Apply");
+    (width - BB - 2 * WB, y, BB, BH, "Generate");
   push[0]->callback(highordertools_runp_cb);
 
   {
@@ -238,40 +233,40 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   {
     y += BH;
     Fl_Box *b = new Fl_Box
-      (x - WB, y, width, BH, "2. Optimization");
+      (x - WB, y, width, BH, "2. Optimization of high order elements");
     b->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
   }
 
   y += BH;
 
   value[1] = new Fl_Value_Input
-    (x,y, IW/2.0, BH);
+    (x, y, IW/2.0, BH);
   value[1]->minimum(0);
   value[1]->maximum(1);
   value[1]->step(.01);
   value[1]->align(FL_ALIGN_RIGHT);
-  value[1]->value(0.1);
+  value[1]->value(CTX::instance()->mesh.hoThresholdMin);
 
   value[8] = new Fl_Value_Input
-    (x+IW/2.0,y, IW/2.0, BH, "Jacobian range");
+    (x+IW/2.0,y, IW/2.0, BH, "Target Jacobian range");
   value[8]->minimum(1);
   value[8]->maximum(10);
   value[8]->step(.01);
   value[8]->align(FL_ALIGN_RIGHT);
-  value[8]->value(2);
+  value[8]->value(CTX::instance()->mesh.hoThresholdMax);
 
   y += BH;
   value[2] = new Fl_Value_Input
-    (x,y, IW, BH, "Number of layers");
+    (x, y, IW, BH, "Number of layers");
   value[2]->minimum(1);
   value[2]->maximum(20);
   value[2]->step(1);
   value[2]->align(FL_ALIGN_RIGHT);
-  value[2]->value(6);
+  value[2]->value(CTX::instance()->mesh.hoNLayers);
 
   y += BH;
   value[7] = new Fl_Value_Input
-    (x,y, IW, BH, "Distance factor");
+    (x, y, IW, BH, "Distance factor");
   value[7]->minimum(1);
   value[7]->maximum(20000);
   value[7]->step(1);
@@ -286,7 +281,7 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
 
   y += BH;
   choice[2] = new Fl_Choice
-    (x,y, IW, BH, "Algorithm");
+    (x, y, IW, BH, "Algorithm");
   choice[2]->align(FL_ALIGN_RIGHT);
   choice[2]->menu(menu_method);
   choice[2]->callback(chooseopti_cb);
@@ -299,24 +294,23 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
 
   y += BH;
   choice[0] = new Fl_Choice
-    (x,y, IW, BH, "Boundary nodes");
+    (x, y, IW, BH, "Boundary vertices");
   choice[0]->menu(menu_objf);
   choice[0]->align(FL_ALIGN_RIGHT);
 
   y += BH;
   value[5] = new Fl_Value_Input
-    (x,y, IW, BH, "W fixed");
+    (x, y, IW/2, BH);
   value[5]->align(FL_ALIGN_RIGHT);
   value[5]->value(1.e+5);
-
   value[6] = new Fl_Value_Input
-    (x+IW*1.5,y, IW, BH, "W free");
+    (x+IW/2,y, IW/2, BH, "W fixed / W free");
   value[6]->align(FL_ALIGN_RIGHT);
   value[6]->value(1.e+2);
 
   y += BH;
   value[3] = new Fl_Value_Input
-    (x,y, IW, BH, "Max. number of iterations");
+    (x, y, IW, BH, "Maximum number of iterations");
   value[3]->minimum(1);
   value[3]->maximum(10000);
   value[3]->step(10);
@@ -325,7 +319,7 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
 
   y += BH;
   value[4] = new Fl_Value_Input
-    (x,y, IW, BH, "Max. number of optimization passes");
+    (x, y, IW, BH, "Max. number of optimization passes");
   value[4]->minimum(1);
   value[4]->maximum(100);
   value[4]->step(1);
@@ -340,14 +334,14 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
 
   y += BH;
   choice[3] = new Fl_Choice
-    (x,y, IW, BH, "Strategy");
+    (x, y, IW, BH, "Strategy");
   choice[3]->menu(menu_strategy);
   choice[3]->align(FL_ALIGN_RIGHT);
   choice[3]->callback(chooseopti_strategy);
 
   y += BH;
   value[9] = new Fl_Value_Input
-    (x,y, IW, BH, "Max. number of blob adaptation iter.");
+    (x, y, IW, BH, "Max. number of blob adaptation iter.");
   value[9]->minimum(1);
   value[9]->maximum(100);
   value[9]->step(1);
@@ -357,7 +351,7 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
 
   y += BH;
   value[10] = new Fl_Value_Input
-    (x,y, IW, BH, "Num. layer adaptation factor");
+    (x, y, IW, BH, "Num. layer adaptation factor");
   value[10]->align(FL_ALIGN_RIGHT);
   value[10]->minimum(1);
   value[10]->maximum(100);
@@ -367,7 +361,7 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
 
   y += BH;
   value[11] = new Fl_Value_Input
-    (x,y, IW, BH, "Distance adaptation factor");
+    (x, y, IW, BH, "Distance adaptation factor");
   value[11]->align(FL_ALIGN_RIGHT);
   value[11]->minimum(1.);
   value[11]->maximum(100.);
@@ -376,16 +370,8 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
 
   y += 1.5*BH;
   push[1] = new Fl_Button
-    (x,y, IW, BH, "Apply");
-  push[1]->callback(highordertools_runelas_cb);
-
-  y += 1.5*BH;
-  messages = new Fl_Browser
-    (x,y, width-2*x, height - y - 2*WB);
-  messages->box(FL_THIN_DOWN_BOX);
-  messages->textfont(FL_COURIER);
-  messages->textsize(FL_NORMAL_SIZE - 1);
-  messages->type(FL_MULTI_BROWSER);
+    (width - BB - 2 * WB, y, BB, BH, "Optimize");
+  push[1]->callback(highordertools_runopti_cb);
 
   // win->resizable(o);
   win->position(CTX::instance()->hotPosition[0], CTX::instance()->hotPosition[1]);
@@ -403,11 +389,11 @@ void highOrderToolsWindow::show(bool redrawOnly)
     value[0]->value(meshOrder);
     butt[0]->value(!complete);
     if (CAD) {
-      output[0]->value("Available");
+      box[0]->label("CAD model is available");
       choice[0]->value(1);
     }
     else {
-      output[0]->value("Not Available");
+      box[0]->label("CAD model is not available");
       choice[0]->deactivate();
     }
     win->show();
