@@ -17,6 +17,7 @@
 #include "Numeric.h"
 #include "GaussLegendre1D.h"
 #include "Context.h"
+#include "OS.h"
 
 #if defined(HAVE_MESH)
 #include "meshGFaceOptimize.h"
@@ -173,10 +174,11 @@ void GFace::resetMeshAttributes()
 {
   meshAttributes.recombine = 0;
   meshAttributes.recombineAngle = 45.;
-  meshAttributes.Method = MESH_UNSTRUCTURED;
+  meshAttributes.method = MESH_UNSTRUCTURED;
   meshAttributes.transfiniteArrangement = 0;
   meshAttributes.transfiniteSmoothing = -1;
   meshAttributes.extrude = 0;
+  meshAttributes.reverseMesh = false;
 }
 
 SBoundingBox3d GFace::bounds() const
@@ -299,10 +301,12 @@ std::string GFace::getAdditionalInfoString()
 
   if(meshAttributes.recombine)
     sstream << " recombined";
-  if(meshAttributes.Method == MESH_TRANSFINITE)
+  if(meshAttributes.method == MESH_TRANSFINITE)
     sstream << " transfinite";
   if(meshAttributes.extrude)
     sstream << " extruded";
+  if(meshAttributes.reverseMesh)
+    sstream << " reverse";
 
   return sstream.str();
 }
@@ -346,7 +350,7 @@ void GFace::writeGEO(FILE *fp)
       it != embedded_vertices.end(); it++)
     fprintf(fp, "Point {%d} In Surface {%d};\n", (*it)->tag(), tag());
 
-  if(meshAttributes.Method == MESH_TRANSFINITE){
+  if(meshAttributes.method == MESH_TRANSFINITE){
     fprintf(fp, "Transfinite Surface {%d}", tag());
     if(meshAttributes.corners.size()){
       fprintf(fp, " = {");
@@ -361,6 +365,9 @@ void GFace::writeGEO(FILE *fp)
 
   if(meshAttributes.recombine)
     fprintf(fp, "Recombine Surface {%d};\n", tag());
+
+  if(meshAttributes.reverseMesh)
+    fprintf(fp, "Reverse Surface {%d};\n", tag());
 }
 
 void GFace::computeMeanPlane()
@@ -799,14 +806,15 @@ void GFace::XYZtoUV(double X, double Y, double Z, double &U, double &V,
 {
   const double Precision = onSurface ? 1.e-8 : 1.e-3;
   const int MaxIter = onSurface ? 25 : 10;
-  const int NumInitGuess = 11;
+  const int NumInitGuess = 9;
 
   double Unew = 0., Vnew = 0., err, err2;
   int iter;
   double mat[3][3], jac[3][3];
   double umin, umax, vmin, vmax;
-  double initu[NumInitGuess] = {0.5, 0.6, 0.4, 0.7, 0.3, 0.8, 0.2, 0.9, 0.1, 1., 0.};
-  double initv[NumInitGuess] = {0.5, 0.6, 0.4, 0.7, 0.3, 0.8, 0.2, 0.9, 0.1, 1., 0.};
+  // don't use 0.9, 0.1 it fails with ruled surfaces
+  double initu[NumInitGuess] = {0.5, 0.6, 0.4, 0.7, 0.3, 0.8, 0.2, 1.0, 0.0};
+  double initv[NumInitGuess] = {0.5, 0.6, 0.4, 0.7, 0.3, 0.8, 0.2, 1.0, 0.0};
 
   Range<double> ru = parBounds(0);
   Range<double> rv = parBounds(1);
@@ -815,10 +823,10 @@ void GFace::XYZtoUV(double X, double Y, double Z, double &U, double &V,
   vmin = rv.low();
   vmax = rv.high();
 
-  const double tol = Precision*(SQU(umax-umin)+SQU(vmax-vmin));
+  const double tol = Precision * (SQU(umax - umin) + SQU(vmax-vmin));
   for(int i = 0; i < NumInitGuess; i++) {
-    initu[i] = umin + initu[i]*(umax-umin);
-    initv[i] = vmin + initv[i]*(vmax-vmin);
+    initu[i] = umin + initu[i] * (umax - umin);
+    initv[i] = vmin + initv[i] * (vmax - vmin);
   }
 
   for(int i = 0; i < NumInitGuess; i++){
@@ -854,10 +862,13 @@ void GFace::XYZtoUV(double X, double Y, double Z, double &U, double &V,
           (jac[0][1] * (X - P.x()) + jac[1][1] * (Y - P.y()) +
            jac[2][1] * (Z - P.z()));
 
-        //if(Unew > umax || Vnew > vmax || Unew < umin || Vnew < vmin) break;
+        // don't remove this test: it is important
+        if((Unew > umax+tol || Unew < umin-tol) &&
+           (Vnew > vmax+tol || Vnew < vmin-tol)) break;
 
         err = SQU(Unew - U) + SQU(Vnew - V);
         err2 = sqrt(SQU(X - P.x()) + SQU(Y - P.y()) + SQU(Z - P.z()));
+
         iter++;
         U = Unew;
         V = Vnew;
@@ -959,7 +970,7 @@ GPoint GFace::closestPoint(const SPoint3 &queryPoint, const double initialGuess[
     printf("dist = %12.5E\n",dist);
   }
 
-  // FILE *F = fopen ("hop.pos","w");
+  // FILE *F = Fopen ("hop.pos","w");
   // fprintf(F,"View \" \" {\n");
   // fprintf(F,"SP(%g,%g,%g) {%g};\n",queryPoint.x(),queryPoint.y(),queryPoint.z(),0.0);
   double initial_guesses = 10.0;
@@ -1355,9 +1366,9 @@ void GFace::lloyd(int nbiter, int infn)
 #endif
 }
 
-void GFace::replaceEdges (std::list<GEdge*> &new_edges)
+void GFace::replaceEdges(std::list<GEdge*> &new_edges)
 {
-  replaceEdgesInternal (new_edges);
+  replaceEdgesInternal(new_edges);
   std::list<GEdge*>::iterator it  = l_edges.begin();
   std::list<GEdge*>::iterator it2 = new_edges.begin();
   std::list<int>::iterator it3 = l_dirs.begin();
@@ -1399,7 +1410,7 @@ void GFace::addLayersOfQuads(int nLayers, GVertex *gv, double hmin, double ratio
 {
   SVector3 ez (0, 0, 1);
   std::list<GEdgeLoop>::iterator it = edgeLoops.begin();
-  FILE *f = fopen ("coucou.pos","w");
+  FILE *f = Fopen ("coucou.pos","w");
   fprintf(f,"View \"\"{\n");
   for (; it != edgeLoops.end(); ++it){
     bool found = false;
