@@ -6,6 +6,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "GModel.h"
+#include "OS.h"
 #include "MLine.h"
 #include "MTriangle.h"
 #include "MQuadrangle.h"
@@ -29,20 +30,21 @@ static bool getVertices(int num, int *indices, std::vector<MVertex*> &vec,
 
 int GModel::readMESH(const std::string &name)
 {
-  FILE *fp = fopen(name.c_str(), "r");
+  FILE *fp = Fopen(name.c_str(), "r");
   if(!fp){
     Msg::Error("Unable to open file '%s'", name.c_str());
     return 0;
   }
 
   char buffer[256];
-  if(!fgets(buffer, sizeof(buffer), fp)) return 0;
+  if(!fgets(buffer, sizeof(buffer), fp)){ fclose(fp); return 0; }
 
   char str[256];
   int format;
   sscanf(buffer, "%s %d", str, &format);
   if(format == 3){
     Msg::Error("Medit mesh import only available for ASCII files");
+    fclose(fp);
     return 0;
   }
 
@@ -82,7 +84,7 @@ int GModel::readMESH(const std::string &name)
           sscanf(buffer, "%d %d %d", &n[0], &n[1], &cl);
           for(int j = 0; j < 2; j++) n[j]--;
           std::vector<MVertex*> vertices;
-          if(!getVertices(2, n, vertexVector, vertices)) return 0;
+          if(!getVertices(2, n, vertexVector, vertices)){ fclose(fp); return 0; }
           elements[0][cl].push_back(new MLine(vertices));
         }
       }
@@ -97,7 +99,7 @@ int GModel::readMESH(const std::string &name)
           sscanf(buffer, "%d %d %d %d", &n[0], &n[1], &n[2], &cl);
           for(int j = 0; j < 3; j++) n[j]--;
           std::vector<MVertex*> vertices;
-          if(!getVertices(3, n, vertexVector, vertices)) return 0;
+          if(!getVertices(3, n, vertexVector, vertices)){ fclose(fp); return 0; }
           elements[0][cl].push_back(new MLine3(vertices));
         }
       }
@@ -112,7 +114,7 @@ int GModel::readMESH(const std::string &name)
           sscanf(buffer, "%d %d %d %d", &n[0], &n[1], &n[2], &cl);
           for(int j = 0; j < 3; j++) n[j]--;
           std::vector<MVertex*> vertices;
-          if(!getVertices(3, n, vertexVector, vertices)) return 0;
+          if(!getVertices(3, n, vertexVector, vertices)){ fclose(fp); return 0; }
           elements[1][cl].push_back(new MTriangle(vertices));
         }
       }
@@ -124,10 +126,11 @@ int GModel::readMESH(const std::string &name)
         for(int i = 0; i < nbe; i++) {
           if(!fgets(buffer, sizeof(buffer), fp)) break;
           int n[6], cl;
-          sscanf(buffer, "%d %d %d %d %d %d %d", &n[0], &n[1], &n[2], &n[3], &n[4], &n[5],&cl);
+          sscanf(buffer, "%d %d %d %d %d %d %d",
+                 &n[0], &n[1], &n[2], &n[3], &n[4], &n[5], &cl);
           for(int j = 0; j < 6; j++) n[j]--;
           std::vector<MVertex*> vertices;
-          if(!getVertices(6, n, vertexVector, vertices)) return 0;
+          if(!getVertices(6, n, vertexVector, vertices)){ fclose(fp); return 0; }
           elements[1][cl].push_back(new MTriangle6(vertices));
         }
       }
@@ -142,7 +145,7 @@ int GModel::readMESH(const std::string &name)
           sscanf(buffer, "%d %d %d %d %d", &n[0], &n[1], &n[2], &n[3], &cl);
           for(int j = 0; j < 4; j++) n[j]--;
           std::vector<MVertex*> vertices;
-          if(!getVertices(4, n, vertexVector, vertices)) return 0;
+          if(!getVertices(4, n, vertexVector, vertices)){ fclose(fp); return 0; }
           elements[2][cl].push_back(new MQuadrangle(vertices));
         }
       }
@@ -157,7 +160,7 @@ int GModel::readMESH(const std::string &name)
           sscanf(buffer, "%d %d %d %d %d", &n[0], &n[1], &n[2], &n[3], &cl);
           for(int j = 0; j < 4; j++) n[j]--;
           std::vector<MVertex*> vertices;
-          if(!getVertices(4, n, vertexVector, vertices)) return 0;
+          if(!getVertices(4, n, vertexVector, vertices)){ fclose(fp); return 0; }
           elements[3][cl].push_back(new MTetrahedron(vertices));
         }
       }
@@ -169,10 +172,11 @@ int GModel::readMESH(const std::string &name)
         for(int i = 0; i < nbe; i++) {
           if(!fgets(buffer, sizeof(buffer), fp)) break;
           int n[10], cl;
-          sscanf(buffer, "%d %d %d %d %d %d %d %d %d %d %d", &n[0], &n[1], &n[2], &n[3],&n[4], &n[5], &n[6], &n[7], &n[9], &n[8], &cl);
+          sscanf(buffer, "%d %d %d %d %d %d %d %d %d %d %d", &n[0], &n[1], &n[2],
+                 &n[3],&n[4], &n[5], &n[6], &n[7], &n[9], &n[8], &cl);
           for(int j = 0; j < 10; j++) n[j]--;
           std::vector<MVertex*> vertices;
-          if(!getVertices(10, n, vertexVector, vertices)) return 0;
+          if(!getVertices(10, n, vertexVector, vertices)){ fclose(fp); return 0; }
           elements[3][cl].push_back(new MTetrahedron10(vertices));
         }
       }
@@ -188,7 +192,7 @@ int GModel::readMESH(const std::string &name)
                  &n[4], &n[5], &n[6], &n[7], &cl);
           for(int j = 0; j < 8; j++) n[j]--;
           std::vector<MVertex*> vertices;
-          if(!getVertices(8, n, vertexVector, vertices)) return 0;
+          if(!getVertices(8, n, vertexVector, vertices)){ fclose(fp); return 0; }
           elements[4][cl].push_back(new MHexahedron(vertices));
         }
       }
@@ -207,7 +211,7 @@ int GModel::readMESH(const std::string &name)
 int GModel::writeMESH(const std::string &name, int elementTagType,
                       bool saveAll, double scalingFactor)
 {
-  FILE *fp = fopen(name.c_str(), "w");
+  FILE *fp = Fopen(name.c_str(), "w");
   if(!fp){
     Msg::Error("Unable to open file '%s'", name.c_str());
     return 0;
