@@ -698,6 +698,35 @@ GRegion* OCC_Internals::addRegionToModel(GModel *model, TopoDS_Solid region)
   return getOCCRegionByNativePtr(model, region);
 }
 
+/* I needed getGTagOfOCC*ByNativePtr whithin setPhysicalNumToEntitiesInBox */
+int OCC_Internals::getGTagOfOCCVertexByNativePtr(GModel *model, TopoDS_Vertex toFind)
+{
+	if(gvNumCache.IsBound(toFind))
+		return (int)gvNumCache.Find(toFind);
+	return 0;	
+}
+
+int OCC_Internals::getGTagOfOCCFaceByNativePtr(GModel *model, TopoDS_Face toFind)
+{
+	if(gfNumCache.IsBound(toFind))
+		return (int)gfNumCache.Find(toFind);
+	return 0;
+}
+
+int OCC_Internals::getGTagOfOCCEdgeByNativePtr(GModel *model, TopoDS_Edge toFind)
+{
+	if(geNumCache.IsBound(toFind))
+		return (int)geNumCache.Find(toFind);
+	return 0;
+}
+
+int OCC_Internals::getGTagOfOCCSolidByNativePtr(GModel *model, TopoDS_Solid toFind)
+{
+	if(grNumCache.IsBound(toFind))
+		return (int)grNumCache.Find(toFind);
+	return 0;	
+}
+
 void OCC_Internals::buildGModel(GModel *model)
 {
   // building geom vertices
@@ -996,6 +1025,7 @@ int GModel::readOCCSTEP(const std::string &fn)
   _occ_internals = new OCC_Internals;
   _occ_internals->loadSTEP(fn.c_str());
   _occ_internals->buildGModel(this);
+  //snapVertices();
   return 1;
 }
 
