@@ -5,6 +5,7 @@
 
 #include <stdio.h>
 #include "GModel.h"
+#include "OS.h"
 #include "MLine.h"
 #include "MTriangle.h"
 #include "MQuadrangle.h"
@@ -14,7 +15,7 @@
 
 int GModel::readSTL(const std::string &name, double tolerance)
 {
-  FILE *fp = fopen(name.c_str(), "rb");
+  FILE *fp = Fopen(name.c_str(), "rb");
   if(!fp){
     Msg::Error("Unable to open file '%s'", name.c_str());
     return 0;
@@ -26,7 +27,7 @@ int GModel::readSTL(const std::string &name, double tolerance)
 
   // "solid", or binary data header
   char buffer[256];
-  if(!fgets(buffer, sizeof(buffer), fp)) return 0;
+  if(!fgets(buffer, sizeof(buffer), fp)){ fclose(fp); return 0; }
 
   bool binary = strncmp(buffer, "solid", 5);
 
@@ -117,11 +118,13 @@ int GModel::readSTL(const std::string &name, double tolerance)
   for(unsigned int i = 0; i < points.size(); i++){
     if(points[i].empty()){
       Msg::Error("No facets found in STL file for solid %d", i);
+      fclose(fp);
       return 0;
     }
     if(points[i].size() % 3){
       Msg::Error("Wrong number of points (%d) in STL file for solid %d",
                  points[i].size(), i);
+      fclose(fp);
       return 0;
     }
     Msg::Info("%d facets in solid %d", points[i].size() / 3, i);
@@ -174,7 +177,7 @@ int GModel::readSTL(const std::string &name, double tolerance)
 int GModel::writeSTL(const std::string &name, bool binary, bool saveAll,
                      double scalingFactor)
 {
-  FILE *fp = fopen(name.c_str(), binary ? "wb" : "w");
+  FILE *fp = Fopen(name.c_str(), binary ? "wb" : "w");
   if(!fp){
     Msg::Error("Unable to open file '%s'", name.c_str());
     return 0;
