@@ -67,11 +67,12 @@ class MPyramid : public MElement {
   virtual int getDim() const { return 3; }
   virtual int getNumVertices() const { return 5; }
   virtual MVertex *getVertex(int num){ return _v[num]; }
+  virtual const MVertex *getVertex(int num) const{ return _v[num]; }
   virtual void setVertex(int num,  MVertex *v){ _v[num] = v; }
   virtual const nodalBasis* getFunctionSpace(int o=-1) const;
   virtual const JacobianBasis* getJacobianFuncSpace(int o=-1) const;
   virtual int getNumEdges(){ return 8; }
-  virtual MEdge getEdge(int num)
+  virtual MEdge getEdge(int num) const
   {
     return MEdge(_v[edges_pyramid(num, 0)], _v[edges_pyramid(num, 1)]);
   }
@@ -120,12 +121,12 @@ class MPyramid : public MElement {
   virtual int getTypeForVTK() const { return 14; }
   virtual const char *getStringForPOS() const { return "SY"; }
   virtual const char *getStringForBDF() const { return "CPYRAM"; }
-  virtual void revert()
+  virtual void reverse()
   {
     MVertex *tmp = _v[0]; _v[0] = _v[2]; _v[2] = tmp;
   }
   virtual int getVolumeSign();
-  virtual void getNode(int num, double &u, double &v, double &w)
+  virtual void getNode(int num, double &u, double &v, double &w) const
   {
     switch(num) {
     case 0 : u = -1.; v = -1.; w = 0.; break;
@@ -136,11 +137,11 @@ class MPyramid : public MElement {
     default: u =  0.; v =  0.; w = 0.; break;
     }
   }
-  virtual SPoint3 barycenterUVW()
+  virtual SPoint3 barycenterUVW() const
   {
     return SPoint3(0., 0., .2);
   }
-  virtual bool isInside(double u, double v, double w)
+  virtual bool isInside(double u, double v, double w) const
   {
     double tol = _isInsideTolerance;
     if(u < (w - (1. + tol)) || u > ((1. + tol) - w) || v < (w - (1. + tol)) ||
@@ -148,6 +149,7 @@ class MPyramid : public MElement {
       return false;
     return true;
   }
+  void getIntegrationPoints(int pOrder, int *npts, IntPt **pts);
   static int edges_pyramid(const int edge, const int vert)
   {
     static const int e[8][2] = {
@@ -165,11 +167,12 @@ class MPyramid : public MElement {
   static int faces_pyramid(const int face, const int vert)
   {
     // only triangular faces
-    static const int f[4][3] = {
-      {0, 1, 4},
-      {3, 0, 4},
-      {1, 2, 4},
-      {2, 3, 4}
+    static const int f[5][4] = {
+      {0, 1, 4, -1},
+      {3, 0, 4, -1},
+      {1, 2, 4, -1},
+      {2, 3, 4, -1},
+      {0, 3, 2, 1}
     };
     return f[face][vert];
   }
@@ -244,6 +247,7 @@ class MPyramidN : public MPyramid {
   virtual int getPolynomialOrder() const { return _order; }
   virtual int getNumVertices() const { return 5 + _vs.size(); }
   virtual MVertex *getVertex(int num){ return num < 5 ? _v[num] : _vs[num - 5]; }
+  virtual const MVertex *getVertex(int num) const{ return num < 5 ? _v[num] : _vs[num - 5]; }
   virtual int getNumEdgeVertices() const { return 8 * (_order - 1); }
   virtual int getNumFaceVertices() const
   {
@@ -290,23 +294,23 @@ class MPyramidN : public MPyramid {
   {
     if(_order == 2 && _vs.size() + 5 == 13) return MSH_PYR_13;
     if(_order == 2 && _vs.size() + 5 == 14) return MSH_PYR_14;
-    if(_order == 3 && _vs.size() + 5 == 29) return MSH_PYR_29;
+    if(_order == 3 && _vs.size() + 5 == 21) return MSH_PYR_21;
     if(_order == 3 && _vs.size() + 5 == 30) return MSH_PYR_30;
-    if(_order == 4 && _vs.size() + 5 == 50) return MSH_PYR_50;
+    if(_order == 4 && _vs.size() + 5 == 29) return MSH_PYR_29;
     if(_order == 4 && _vs.size() + 5 == 55) return MSH_PYR_55;
-    if(_order == 5 && _vs.size() + 5 == 77) return MSH_PYR_77;
+    if(_order == 5 && _vs.size() + 5 == 37) return MSH_PYR_37;
     if(_order == 5 && _vs.size() + 5 == 91) return MSH_PYR_91;
-    if(_order == 6 && _vs.size() + 5 == 110) return MSH_PYR_110;
+    if(_order == 6 && _vs.size() + 5 == 45) return MSH_PYR_45;
     if(_order == 6 && _vs.size() + 5 == 140) return MSH_PYR_140;
-    if(_order == 7 && _vs.size() + 5 == 149) return MSH_PYR_149;
+    if(_order == 7 && _vs.size() + 5 == 53)  return MSH_PYR_53;
     if(_order == 7 && _vs.size() + 5 == 204) return MSH_PYR_204;
-    if(_order == 8 && _vs.size() + 5 == 194) return MSH_PYR_194;
+    if(_order == 8 && _vs.size() + 5 == 61)  return MSH_PYR_61;
     if(_order == 8 && _vs.size() + 5 == 285) return MSH_PYR_285;
-    if(_order == 9 && _vs.size() + 5 == 245) return MSH_PYR_245;
+    if(_order == 9 && _vs.size() + 5 == 69)  return MSH_PYR_69;
     if(_order == 9 && _vs.size() + 5 == 385) return MSH_PYR_385;
     return 0;
   }
-  virtual void revert()
+  virtual void reverse()
   {
 /*    MVertex *tmp;
     tmp = _v[1]; _v[1] = _v[2]; _v[2] = tmp;
@@ -316,14 +320,14 @@ class MPyramidN : public MPyramid {
       inv[i] = _vs[reverseIndices[i + 4] - 4];
     _vs = inv;*/
 
-    Msg::Error("Revert not implemented yet for MPyramidN");
+    Msg::Error("Reverse not implemented yet for MPyramidN");
 
   }
   virtual void getEdgeRep(int num, double *x, double *y, double *z, SVector3 *n);
   virtual int getNumEdgesRep();
   virtual void getFaceRep(int num, double *x, double *y, double *z, SVector3 *n);
   virtual int getNumFacesRep();
-  virtual void getNode(int num, double &u, double &v, double &w)
+  virtual void getNode(int num, double &u, double &v, double &w) const
   {
     num < 5 ? MPyramid::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
   }
