@@ -92,7 +92,7 @@ double MTriangle::gammaShapeMeasure()
 #endif
 }
 
- void MTriangle::xyz2uvw(double xyz[3], double uvw[3])
+ void MTriangle::xyz2uvw(double xyz[3], double uvw[3]) const
  {
    const double O[3] = {_v[0]->x(), _v[0]->y(), _v[0]->z()};
    const double d[3] = {xyz[0] - O[0], xyz[1] - O[1], xyz[2] - O[2]};
@@ -127,33 +127,33 @@ const nodalBasis* MTriangle::getFunctionSpace(int o) const
 
   if ((nf == 0) && (o == -1)) {
     switch (order) {
-    case 0: return BasisFactory::create(MSH_TRI_1);
-    case 1: return BasisFactory::create(MSH_TRI_3);
-    case 2: return BasisFactory::create(MSH_TRI_6);
-    case 3: return BasisFactory::create(MSH_TRI_9);
-    case 4: return BasisFactory::create(MSH_TRI_12);
-    case 5: return BasisFactory::create(MSH_TRI_15I);
-    case 6: return BasisFactory::create(MSH_TRI_18);
-    case 7: return BasisFactory::create(MSH_TRI_21I);
-    case 8: return BasisFactory::create(MSH_TRI_24);
-    case 9: return BasisFactory::create(MSH_TRI_27);
-    case 10: return BasisFactory::create(MSH_TRI_30);
+    case 0: return BasisFactory::getNodalBasis(MSH_TRI_1);
+    case 1: return BasisFactory::getNodalBasis(MSH_TRI_3);
+    case 2: return BasisFactory::getNodalBasis(MSH_TRI_6);
+    case 3: return BasisFactory::getNodalBasis(MSH_TRI_9);
+    case 4: return BasisFactory::getNodalBasis(MSH_TRI_12);
+    case 5: return BasisFactory::getNodalBasis(MSH_TRI_15I);
+    case 6: return BasisFactory::getNodalBasis(MSH_TRI_18);
+    case 7: return BasisFactory::getNodalBasis(MSH_TRI_21I);
+    case 8: return BasisFactory::getNodalBasis(MSH_TRI_24);
+    case 9: return BasisFactory::getNodalBasis(MSH_TRI_27);
+    case 10: return BasisFactory::getNodalBasis(MSH_TRI_30);
     default: Msg::Error("Order %d triangle incomplete function space not implemented", order);
     }
   }
   else {
     switch (order) {
-    case 0: return BasisFactory::create(MSH_TRI_1);
-    case 1: return BasisFactory::create(MSH_TRI_3);
-    case 2: return BasisFactory::create(MSH_TRI_6);
-    case 3: return BasisFactory::create(MSH_TRI_10);
-    case 4: return BasisFactory::create(MSH_TRI_15);
-    case 5: return BasisFactory::create(MSH_TRI_21);
-    case 6: return BasisFactory::create(MSH_TRI_28);
-    case 7: return BasisFactory::create(MSH_TRI_36);
-    case 8: return BasisFactory::create(MSH_TRI_45);
-    case 9: return BasisFactory::create(MSH_TRI_55);
-    case 10: return BasisFactory::create(MSH_TRI_66);
+    case 0: return BasisFactory::getNodalBasis(MSH_TRI_1);
+    case 1: return BasisFactory::getNodalBasis(MSH_TRI_3);
+    case 2: return BasisFactory::getNodalBasis(MSH_TRI_6);
+    case 3: return BasisFactory::getNodalBasis(MSH_TRI_10);
+    case 4: return BasisFactory::getNodalBasis(MSH_TRI_15);
+    case 5: return BasisFactory::getNodalBasis(MSH_TRI_21);
+    case 6: return BasisFactory::getNodalBasis(MSH_TRI_28);
+    case 7: return BasisFactory::getNodalBasis(MSH_TRI_36);
+    case 8: return BasisFactory::getNodalBasis(MSH_TRI_45);
+    case 9: return BasisFactory::getNodalBasis(MSH_TRI_55);
+    case 10: return BasisFactory::getNodalBasis(MSH_TRI_66);
     default: Msg::Error("Order %d triangle function space not implemented", order);
     }
   }
@@ -169,32 +169,32 @@ const JacobianBasis* MTriangle::getJacobianFuncSpace(int o) const
 
   if ((nf == 0) && (o == -1)) {
     switch (order) {
-    case 0: return JacobianBasis::find(MSH_TRI_1);
-    case 1: return JacobianBasis::find(MSH_TRI_3);
-    case 2: return JacobianBasis::find(MSH_TRI_6);
-    case 3: return JacobianBasis::find(MSH_TRI_9);
-    case 4: return JacobianBasis::find(MSH_TRI_12);
-    case 5: return JacobianBasis::find(MSH_TRI_15I);
-    case 6: return JacobianBasis::find(MSH_TRI_18);
-    case 7: return JacobianBasis::find(MSH_TRI_21I);
-    case 8: return JacobianBasis::find(MSH_TRI_24);
-    case 9: return JacobianBasis::find(MSH_TRI_27);
-    case 10: return JacobianBasis::find(MSH_TRI_30);
+    case 0: return BasisFactory::getJacobianBasis(MSH_TRI_1);
+    case 1: return BasisFactory::getJacobianBasis(MSH_TRI_3);
+    case 2: return BasisFactory::getJacobianBasis(MSH_TRI_6);
+    case 3: return BasisFactory::getJacobianBasis(MSH_TRI_9);
+    case 4: return BasisFactory::getJacobianBasis(MSH_TRI_12);
+    case 5: return BasisFactory::getJacobianBasis(MSH_TRI_15I);
+    case 6: return BasisFactory::getJacobianBasis(MSH_TRI_18);
+    case 7: return BasisFactory::getJacobianBasis(MSH_TRI_21I);
+    case 8: return BasisFactory::getJacobianBasis(MSH_TRI_24);
+    case 9: return BasisFactory::getJacobianBasis(MSH_TRI_27);
+    case 10: return BasisFactory::getJacobianBasis(MSH_TRI_30);
     default: Msg::Error("Order %d triangle incomplete function space not implemented", order);
     }
   }
   else {
     switch (order) {
-    case 1: return JacobianBasis::find(MSH_TRI_3);
-    case 2: return JacobianBasis::find(MSH_TRI_6);
-    case 3: return JacobianBasis::find(MSH_TRI_10);
-    case 4: return JacobianBasis::find(MSH_TRI_15);
-    case 5: return JacobianBasis::find(MSH_TRI_21);
-    case 6: return JacobianBasis::find(MSH_TRI_28);
-    case 7: return JacobianBasis::find(MSH_TRI_36);
-    case 8: return JacobianBasis::find(MSH_TRI_45);
-    case 9: return JacobianBasis::find(MSH_TRI_55);
-    case 10: return JacobianBasis::find(MSH_TRI_66);
+    case 1: return BasisFactory::getJacobianBasis(MSH_TRI_3);
+    case 2: return BasisFactory::getJacobianBasis(MSH_TRI_6);
+    case 3: return BasisFactory::getJacobianBasis(MSH_TRI_10);
+    case 4: return BasisFactory::getJacobianBasis(MSH_TRI_15);
+    case 5: return BasisFactory::getJacobianBasis(MSH_TRI_21);
+    case 6: return BasisFactory::getJacobianBasis(MSH_TRI_28);
+    case 7: return BasisFactory::getJacobianBasis(MSH_TRI_36);
+    case 8: return BasisFactory::getJacobianBasis(MSH_TRI_45);
+    case 9: return BasisFactory::getJacobianBasis(MSH_TRI_55);
+    case 10: return BasisFactory::getJacobianBasis(MSH_TRI_66);
     default: Msg::Error("Order %d triangle function space not implemented", order);
     }
   }
