@@ -41,7 +41,6 @@ struct Branch{
 };
 
 #if defined(HAVE_ANN)
-#include <ANN/ANN.h>
 class ANNkd_tree;
 
 // This class takes as input A 1D mesh which is the centerline
@@ -58,9 +57,6 @@ class Centerline : public Field{
   GModel *mod; //centerline GModel
   GModel *split; //split GModel
   ANNkd_tree *kdtree, *kdtreeR; 
-  ANNpointArray nodes, nodesR;
-  ANNidxArray index;
-  ANNdistArray dist;
   std::string fileName;
   int nbPoints;
   double recombine;
@@ -121,6 +117,9 @@ class Centerline : public Field{
   //anisotropic operator
   void operator() (double x, double y, double z, SMetric3 &metr, GEntity *ge=0);
 
+  void computeCrossField(double x,double y,double z,
+			 SVector3 &d1, SVector3 &d2,  SVector3 &d3);
+
   //import the 1D mesh of the centerlines (in vtk format)
   //and fill the vector of lines
   void importFile(std::string fileName);
