@@ -25,7 +25,7 @@
 #include "BackgroundMesh.h"
 #include "CenterlineField.h"
 #include "STensor3.h"
-
+#include "meshMetric.h"
 #if defined(HAVE_POST)
 #include "PView.h"
 #include "OctreePost.h"
@@ -239,7 +239,7 @@ class StructuredField : public Field
   }
 };
 
-class UTMField : public Field
+/*class UTMField : public Field
 {
   int iField, zone;
   double a, b, n, n2, n3, n4, n5, e, e2, e1, e12, e13, e14, J1, J2, J3, J4,
@@ -325,7 +325,7 @@ class UTMField : public Field
     if(!field || iField == id) return MAX_LC;
     return (*field)(utm_x, utm_y, 0);
   }
-};
+};*/
 
 class LonLatField : public Field
 {
@@ -371,6 +371,7 @@ class LonLatField : public Field
   }
 };
 
+
 class BoxField : public Field
 {
   double v_in, v_out, x_min, x_max, y_min, y_max, z_min, z_max;
@@ -1704,14 +1705,24 @@ class AttractorField : public Field
           it != edges_id.end(); ++it) {
 	GEdge *e = GModel::current()->getEdgeByTag(*it);
 	if(e) {
-	  for(int i = 1; i < n_nodes_by_edge - 1; i++) {
-	    double u = (double)i / (n_nodes_by_edge - 1);
+	  if (e->mesh_vertices.size()){
+	    for(unsigned int i = 0; i < e->mesh_vertices.size(); i++) {
+	      double u ; e->mesh_vertices[i]->getParameter(0,u);
+	      GPoint gp = e->point(u);
+	      getCoord(gp.x(), gp.y(), gp.z(), zeronodes[k][0],
+		       zeronodes[k][1], zeronodes[k][2], e);
+	      _infos[k++] = AttractorInfo(*it,1,u,0);
+	    }
+	  }
+	  int NNN = n_nodes_by_edge - e->mesh_vertices.size();
+	  for(int i = 1; i < NNN - 1; i++) {
+	    double u = (double)i / (NNN - 1);
 	    Range<double> b = e->parBounds(0);
 	    double t = b.low() + u * (b.high() - b.low());
 	    GPoint gp = e->point(t);
 	    getCoord(gp.x(), gp.y(), gp.z(), zeronodes[k][0],
 		     zeronodes[k][1], zeronodes[k][2], e);
-	    _infos[k++] = AttractorInfo(*it,1,u,0);
+	    _infos[k++] = AttractorInfo(*it,1,t,0);
           }
         }
       }
@@ -1849,6 +1860,41 @@ double BoundaryLayerField::operator() (double x, double y, double z, GEntity *ge
   return std::min (hfar,lc);
 }
 
+// assume that the closest point is one of the model vertices
+void BoundaryLayerField::computeFor1dMesh (double x, double y, double z,
+					   SMetric3 &metr)
+{
+  double xpk = 0., ypk = 0., zpk = 0.;
+  double distk = 1.e22;
+  for(std::list<int>::iterator it = nodes_id.begin();
+      it != nodes_id.end(); ++it) {
+    GVertex *v = GModel::current()->getVertexByTag(*it);
+    double xp = v->x();
+    double yp = v->y();
+    double zp = v->z();
+    const double dist = sqrt ((x - xp) *(x - xp)+
+			      (y - yp) *(y - yp)+
+			      (z - zp) *(z - zp));
+    if (dist < distk){
+      distk = dist;
+      xpk = xp;
+      ypk = yp;
+      zpk = zp;
+    }
+  }
+
+  const double ll1   = (distk*(ratio-1) + hwall_n) / (1. + 0.5 * (ratio - 1));
+  // const double ll1   = (distk*(ratio-1) + hwall_n) / (1.);
+  double lc_n  = std::min(ll1,hfar);
+
+  if (distk > thickness) lc_n = hfar;
+  lc_n = std::max(lc_n, CTX::instance()->mesh.lcMin);
+  lc_n = std::min(lc_n, CTX::instance()->mesh.lcMax);
+  SVector3 t1= SVector3(x-xpk,y-ypk,z-zpk);
+  t1.normalize();
+  metr = buildMetricTangentToCurve(t1,lc_n,lc_n);
+}
+
 void BoundaryLayerField::operator() (AttractorField *cc, double dist,
                                      double x, double y, double z,
                                      SMetric3 &metr, GEntity *ge)
@@ -1944,7 +1990,7 @@ void BoundaryLayerField::operator() (double x, double y, double z,
     }
     for(std::list<int>::iterator it = edges_id.begin();
 	it != edges_id.end(); ++it) {
-      _att_fields.push_back(new AttractorField(1,*it,100000));
+      _att_fields.push_back(new AttractorField(1,*it,10000));
     }
     for(std::list<int>::iterator it = faces_id.begin();
 	it != faces_id.end(); ++it) {
@@ -1995,16 +2041,6 @@ void BoundaryLayerField::operator() (double x, double y, double z,
 }
 #endif
 
-template<class F> class FieldFactoryT : public FieldFactory {
- public:
-  Field * operator()() { return new F; }
-};
-
-template<class F> Field *field_factory()
-{
-  return new F();
-};
-
 FieldManager::FieldManager()
 {
   map_type_name["Structured"] = new FieldFactoryT<StructuredField>();
@@ -2025,7 +2061,6 @@ FieldManager::FieldManager()
   map_type_name["Min"] = new FieldFactoryT<MinField>();
   map_type_name["MinAniso"] = new FieldFactoryT<MinAnisoField>();
   map_type_name["Max"] = new FieldFactoryT<MaxField>();
-  map_type_name["UTM"] = new FieldFactoryT<UTMField>();
   map_type_name["Laplacian"] = new FieldFactoryT<LaplacianField>();
   map_type_name["Mean"] = new FieldFactoryT<MeanField>();
   map_type_name["Curvature"] = new FieldFactoryT<CurvatureField>();
