@@ -156,6 +156,9 @@ class BoundaryLayerField : public Field {
   virtual double operator() (double x, double y, double z, GEntity *ge=0);
   virtual void operator() (double x, double y, double z, SMetric3 &metr, GEntity *ge=0);
   bool isFaceBL (int iF) const {return std::find(faces_id.begin(),faces_id.end(),iF) != faces_id.end();}
+  bool isEdgeBL (int iE) const {return std::find(edges_id.begin(),edges_id.end(),iE) != edges_id.end();}
+  bool isVertexBL (int iV) const {return std::find(nodes_id.begin(),nodes_id.end(),iV) != nodes_id.end();}
+  void computeFor1dMesh(double x, double y, double z, SMetric3 &metr);
 };
 #endif
 class FieldOptionString : public FieldOption
@@ -275,4 +278,9 @@ class FieldCallbackGeneric : public FieldCallback {
   }
 };
 
+template<class F> class FieldFactoryT : public FieldFactory {
+ public:
+  Field * operator()() { return new F; }
+};
+
 #endif
